package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;

import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;

import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.dict.domain.IBZDictItem;
import cn.ibizlab.core.dict.service.IIBZDictItemService;
import cn.ibizlab.core.dict.filter.IBZDictItemSearchContext;




@Slf4j
@Api(tags = {"IBZDictItem" })
@RestController("api-ibzdictitem")
@RequestMapping("")
public class IBZDictItemResource {

    @Autowired
    private IIBZDictItemService ibzdictitemService;

    @Autowired
    @Lazy
    private IBZDictItemMapping ibzdictitemMapping;




    @PreAuthorize("hasPermission(#ibzdictitem_id,'Get',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "Get", tags = {"IBZDictItem" },  notes = "Get")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdictitems/{ibzdictitem_id}")
    public ResponseEntity<IBZDictItemDTO> get(@PathVariable("ibzdictitem_id") String ibzdictitem_id) {
        IBZDictItem domain = ibzdictitemService.get(ibzdictitem_id);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }




    @ApiOperation(value = "GetDraft", tags = {"IBZDictItem" },  notes = "GetDraft")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdictitems/getdraft")
    public ResponseEntity<IBZDictItemDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemMapping.toDto(ibzdictitemService.getDraft(new IBZDictItem())));
    }




    @ApiOperation(value = "Save", tags = {"IBZDictItem" },  notes = "Save")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems/save")
    public ResponseEntity<Boolean> save(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.save(ibzdictitemMapping.toDomain(ibzdictitemdto)));
    }

    @ApiOperation(value = "SaveBatch", tags = {"IBZDictItem" },  notes = "SaveBatch")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        ibzdictitemService.saveBatch(ibzdictitemMapping.toDomain(ibzdictitemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }




    @ApiOperation(value = "CheckKey", tags = {"IBZDictItem" },  notes = "CheckKey")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.checkKey(ibzdictitemMapping.toDomain(ibzdictitemdto)));
    }




    @PreAuthorize("hasPermission('Remove',{#ibzdictitem_id,{this.getEntity(),'Sql'}})")
    @ApiOperation(value = "Remove", tags = {"IBZDictItem" },  notes = "Remove")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("ibzdictitem_id") String ibzdictitem_id) {
         return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.remove(ibzdictitem_id));
    }

    @ApiOperation(value = "RemoveBatch", tags = {"IBZDictItem" },  notes = "RemoveBatch")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdictitems/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        ibzdictitemService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }




    @PreAuthorize("hasPermission('','Create',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "Create", tags = {"IBZDictItem" },  notes = "Create")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> create(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
		ibzdictitemService.create(domain);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission('','Create',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "createBatch", tags = {"IBZDictItem" },  notes = "createBatch")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        ibzdictitemService.createBatch(ibzdictitemMapping.toDomain(ibzdictitemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }




    @PreAuthorize("hasPermission(#ibzdictitem_id,'Update',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "Update", tags = {"IBZDictItem" },  notes = "Update")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> update(@PathVariable("ibzdictitem_id") String ibzdictitem_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
		IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
        domain.setItemid(ibzdictitem_id);
		ibzdictitemService.update(domain);
		IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(#ibzdictitem_id,'Update',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "UpdateBatch", tags = {"IBZDictItem" },  notes = "UpdateBatch")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdictitems/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        ibzdictitemService.updateBatch(ibzdictitemMapping.toDomain(ibzdictitemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission('Get',{#context,'Default',this.getEntity(),'Sql'})")
	@ApiOperation(value = "fetchDEFAULT", tags = {"IBZDictItem" } ,notes = "fetchDEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdictitems/fetchdefault")
	public ResponseEntity<List<IBZDictItemDTO>> fetchDefault(IBZDictItemSearchContext context) {
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
        List<IBZDictItemDTO> list = ibzdictitemMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasPermission('Get',{#context,'Default',this.getEntity(),'Sql'})")
	@ApiOperation(value = "searchDEFAULT", tags = {"IBZDictItem" } ,notes = "searchDEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdictitems/searchdefault")
	public ResponseEntity<Page<IBZDictItemDTO>> searchDefault(IBZDictItemSearchContext context) {
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(ibzdictitemMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}



    @ApiOperation(value = "GetByIBZDict", tags = {"IBZDictItem" },  notes = "GetByIBZDict")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    public ResponseEntity<IBZDictItemDTO> getByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id) {
        IBZDictItem domain = ibzdictitemService.get(ibzdictitem_id);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "GetDraftByIBZDict", tags = {"IBZDictItem" },  notes = "GetDraftByIBZDict")
    @RequestMapping(method = RequestMethod.GET, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/getdraft")
    public ResponseEntity<IBZDictItemDTO> getDraftByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id) {
        IBZDictItem domain = new IBZDictItem();
        domain.setDictid(ibzdict_id);
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemMapping.toDto(ibzdictitemService.getDraft(domain)));
    }

    @ApiOperation(value = "SaveByIBZDict", tags = {"IBZDictItem" },  notes = "SaveByIBZDict")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/save")
    public ResponseEntity<Boolean> saveByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
        domain.setDictid(ibzdict_id);
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.save(domain));
    }

    @ApiOperation(value = "SaveBatchByIBZDict", tags = {"IBZDictItem" },  notes = "SaveBatchByIBZDict")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/savebatch")
    public ResponseEntity<Boolean> saveBatchByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        List<IBZDictItem> domainlist=ibzdictitemMapping.toDomain(ibzdictitemdtos);
        for(IBZDictItem domain:domainlist){
             domain.setDictid(ibzdict_id);
        }
        ibzdictitemService.saveBatch(domainlist);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "CheckKeyByIBZDict", tags = {"IBZDictItem" },  notes = "CheckKeyByIBZDict")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/checkkey")
    public ResponseEntity<Boolean> checkKeyByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.checkKey(ibzdictitemMapping.toDomain(ibzdictitemdto)));
    }

    @ApiOperation(value = "RemoveByIBZDict", tags = {"IBZDictItem" },  notes = "RemoveByIBZDict")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<Boolean> removeByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id) {
		return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.remove(ibzdictitem_id));
    }

    @ApiOperation(value = "RemoveBatchByIBZDict", tags = {"IBZDictItem" },  notes = "RemoveBatchByIBZDict")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/batch")
    public ResponseEntity<Boolean> removeBatchByIBZDict(@RequestBody List<String> ids) {
        ibzdictitemService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "CreateByIBZDict", tags = {"IBZDictItem" },  notes = "CreateByIBZDict")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> createByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
        domain.setDictid(ibzdict_id);
		ibzdictitemService.create(domain);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "createBatchByIBZDict", tags = {"IBZDictItem" },  notes = "createBatchByIBZDict")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/batch")
    public ResponseEntity<Boolean> createBatchByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        List<IBZDictItem> domainlist=ibzdictitemMapping.toDomain(ibzdictitemdtos);
        for(IBZDictItem domain:domainlist){
            domain.setDictid(ibzdict_id);
        }
        ibzdictitemService.createBatch(domainlist);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "UpdateByIBZDict", tags = {"IBZDictItem" },  notes = "UpdateByIBZDict")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> updateByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
        domain.setDictid(ibzdict_id);
        domain.setItemid(ibzdictitem_id);
		ibzdictitemService.update(domain);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "UpdateBatchByIBZDict", tags = {"IBZDictItem" },  notes = "UpdateBatchByIBZDict")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/batch")
    public ResponseEntity<Boolean> updateBatchByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        List<IBZDictItem> domainlist=ibzdictitemMapping.toDomain(ibzdictitemdtos);
        for(IBZDictItem domain:domainlist){
            domain.setDictid(ibzdict_id);
        }
        ibzdictitemService.updateBatch(domainlist);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

	@ApiOperation(value = "fetchDEFAULTByIBZDict", tags = {"IBZDictItem" } ,notes = "fetchDEFAULTByIBZDict")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdicts/{ibzdict_id}/ibzdictitems/fetchdefault")
	public ResponseEntity<List<IBZDictItemDTO>> fetchIBZDictItemDefaultByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id,IBZDictItemSearchContext context) {
        context.setN_dictid_eq(ibzdict_id);
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
        List<IBZDictItemDTO> list = ibzdictitemMapping.toDto(domains.getContent());
	    return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

	@ApiOperation(value = "searchDEFAULTByIBZDict", tags = {"IBZDictItem" } ,notes = "searchDEFAULTByIBZDict")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdicts/{ibzdict_id}/ibzdictitems/searchdefault")
	public ResponseEntity<Page<IBZDictItemDTO>> searchIBZDictItemDefaultByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id,IBZDictItemSearchContext context) {
        context.setN_dictid_eq(ibzdict_id);
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(ibzdictitemMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}


    /**
     * 用户权限校验
     * @return
     */
	public IBZDictItem getEntity(){
        return new IBZDictItem();
    }

}
