package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;

import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;

import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.dict.domain.IBZDict;
import cn.ibizlab.core.dict.service.IIBZDictService;
import cn.ibizlab.core.dict.filter.IBZDictSearchContext;




@Slf4j
@Api(tags = {"IBZDict" })
@RestController("api-ibzdict")
@RequestMapping("")
public class IBZDictResource {

    @Autowired
    private IIBZDictService ibzdictService;

    @Autowired
    @Lazy
    private IBZDictMapping ibzdictMapping;




    @PreAuthorize("hasPermission(#ibzdict_id,'Get',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "Get", tags = {"IBZDict" },  notes = "Get")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdicts/{ibzdict_id}")
    public ResponseEntity<IBZDictDTO> get(@PathVariable("ibzdict_id") String ibzdict_id) {
        IBZDict domain = ibzdictService.get(ibzdict_id);
        IBZDictDTO dto = ibzdictMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }




    @PreAuthorize("hasPermission('','Create',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "Create", tags = {"IBZDict" },  notes = "Create")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts")
    @Transactional
    public ResponseEntity<IBZDictDTO> create(@RequestBody IBZDictDTO ibzdictdto) {
        IBZDict domain = ibzdictMapping.toDomain(ibzdictdto);
		ibzdictService.create(domain);
        IBZDictDTO dto = ibzdictMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission('','Create',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "createBatch", tags = {"IBZDict" },  notes = "createBatch")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<IBZDictDTO> ibzdictdtos) {
        ibzdictService.createBatch(ibzdictMapping.toDomain(ibzdictdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }




    @ApiOperation(value = "CheckKey", tags = {"IBZDict" },  notes = "CheckKey")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody IBZDictDTO ibzdictdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(ibzdictService.checkKey(ibzdictMapping.toDomain(ibzdictdto)));
    }




    @ApiOperation(value = "GetDraft", tags = {"IBZDict" },  notes = "GetDraft")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdicts/getdraft")
    public ResponseEntity<IBZDictDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictMapping.toDto(ibzdictService.getDraft(new IBZDict())));
    }




    @PreAuthorize("hasPermission(#ibzdict_id,'Update',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "Update", tags = {"IBZDict" },  notes = "Update")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdicts/{ibzdict_id}")
    @Transactional
    public ResponseEntity<IBZDictDTO> update(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictDTO ibzdictdto) {
		IBZDict domain = ibzdictMapping.toDomain(ibzdictdto);
        domain.setDictid(ibzdict_id);
		ibzdictService.update(domain);
		IBZDictDTO dto = ibzdictMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(#ibzdict_id,'Update',{this.getEntity(),'Sql'})")
    @ApiOperation(value = "UpdateBatch", tags = {"IBZDict" },  notes = "UpdateBatch")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdicts/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<IBZDictDTO> ibzdictdtos) {
        ibzdictService.updateBatch(ibzdictMapping.toDomain(ibzdictdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }




    @PreAuthorize("hasPermission('Remove',{#ibzdict_id,{this.getEntity(),'Sql'}})")
    @ApiOperation(value = "Remove", tags = {"IBZDict" },  notes = "Remove")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdicts/{ibzdict_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("ibzdict_id") String ibzdict_id) {
         return ResponseEntity.status(HttpStatus.OK).body(ibzdictService.remove(ibzdict_id));
    }

    @ApiOperation(value = "RemoveBatch", tags = {"IBZDict" },  notes = "RemoveBatch")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdicts/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        ibzdictService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }




    @ApiOperation(value = "Save", tags = {"IBZDict" },  notes = "Save")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/save")
    public ResponseEntity<Boolean> save(@RequestBody IBZDictDTO ibzdictdto) {
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictService.save(ibzdictMapping.toDomain(ibzdictdto)));
    }

    @ApiOperation(value = "SaveBatch", tags = {"IBZDict" },  notes = "SaveBatch")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<IBZDictDTO> ibzdictdtos) {
        ibzdictService.saveBatch(ibzdictMapping.toDomain(ibzdictdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission('Get',{#context,'Default',this.getEntity(),'Sql'})")
	@ApiOperation(value = "fetchDEFAULT", tags = {"IBZDict" } ,notes = "fetchDEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdicts/fetchdefault")
	public ResponseEntity<List<IBZDictDTO>> fetchDefault(IBZDictSearchContext context) {
        Page<IBZDict> domains = ibzdictService.searchDefault(context) ;
        List<IBZDictDTO> list = ibzdictMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasPermission('Get',{#context,'Default',this.getEntity(),'Sql'})")
	@ApiOperation(value = "searchDEFAULT", tags = {"IBZDict" } ,notes = "searchDEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdicts/searchdefault")
	public ResponseEntity<Page<IBZDictDTO>> searchDefault(IBZDictSearchContext context) {
        Page<IBZDict> domains = ibzdictService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(ibzdictMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}


    /**
     * 用户权限校验
     * @return
     */
	public IBZDict getEntity(){
        return new IBZDict();
    }

}
